package controller;

import config.Config;
import config.subConfig.AxisConfig;
import config.subConfig.DoorConfig;
import config.subConfig.TemplateConfig;
import model.Axis;
import model.Door;
import model.Room;
import model.template.Template;
import utils.TemplateUtils;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * @author duh
 * @create 2018/7/17 16:01
 * @email duh@elab-plus.com
 **/
public class MyPanel extends JPanel{
    private MyFrame frame;
    private Config config;

    public MyPanel(MyFrame frame, Config config) {
        super();
        this.frame = frame;
        this.config = config;
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        AxisConfig axisConfig = config.getAxisPoint();
        int lenght = axisConfig.getLength()+axisConfig.getX()+100;
        int width = axisConfig.getY()+100;
        BufferedImage bufferedImage = new BufferedImage(lenght,
                width, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2d = bufferedImage.createGraphics();
        //画在panel中
        drawContent(g);
        //保存在图片中
        drawContent(g2d);

//        drawTemplateAndMove(room,g);

        try {
            ImageIO.write(bufferedImage, "jpeg", new File("E:\\config\\jpanel.jpg"));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void drawContent(Graphics g) {
        drawAxis(config,g);
        Room room = new Room(config.getRoom());
        drawRoom(room,g);
        drawTemplate(room,g);
        TemplateUtils.setTemplateIndex(0);
    }

    /**
     * 画坐标系
     * @param config
     * @param g
     */
    private void drawAxis(Config config,Graphics g){
        Axis axis = new Axis();
        AxisConfig axisConfig = config.getAxisPoint();
        //ModuleInAxis已设置坐标轴的偏转
        axis.setX(0);
        axis.setY(0);
        axis.setxLength(axisConfig.getLength());
        axis.setyLength(axisConfig.getWidth());
        axis.setxSpacing(axisConfig.getxSpacing());
        axis.setySpacing(axisConfig.getySpacing());
//        System.out.println("axis3 x="+axis.getX());
        axis.drawSelf(g);
    }
    private void drawTemplate(Room room, Graphics g) {
        List<TemplateConfig> templateList = config.getTemplates();
        TemplateUtils.setRoom(room);
        boolean canYizhi = TemplateUtils.checkYiZhiArray(templateList);
        System.out.println("can yi zhi ="+canYizhi);
        if(canYizhi){
            TemplateUtils.calcYizhiTemplatesAndDraw(templateList,g);
        }
//        for(TemplateConfig templateConfig : templateList){
//            Template template = new Template(templateConfig);
//            TemplateUtils.drawTemplate(template,g);
//        }
    }
    private void drawTemplateAndMove(Room room, Graphics g) {
        List<TemplateConfig> templateList = config.getTemplates();
        TemplateUtils.setRoom(room);
        TemplateConfig templateConfig = templateList.get(1);
        Template template = new Template(templateConfig);
        TemplateUtils.drawTemplate(template,g);
        TemplateUtils.routeTemplate(template,3);
        TemplateUtils.drawTemplate(template,g);
    }
    private void drawRoom(Room room, Graphics g){
        room.drawSelf(g);
        DoorConfig doorConfig = config.getDoor();
        Door door = new Door();
        door.setConfig(doorConfig);
        door.setRoom(room);
        door.drawSelf(g);
    }
}
